// Targa.cpp: implementation of the CTarga class.
//
//////////////////////////////////////////////////////////////////////

#include "Targa.h"
#include <windows.h>
#include <cassert>

int CTarga::fgetLittleShort( FILE *f )
{
	byte	b1, b2;

	b1 = fgetc(f);
	b2 = fgetc(f);

	return (short)(b1 + b2*256);
}

int CTarga::fgetLittleLong( FILE *f )
{
	byte	b1, b2, b3, b4;
	b1 = fgetc(f);
	b2 = fgetc(f);
	b3 = fgetc(f);
	b4 = fgetc(f);
	return b1 + (b2<<8) + (b3<<16) + (b4<<24);
}

dword_* CTarga::LoadTGA( string fileName, int& width, int& height )
{
	typedef struct {
		unsigned char 	id_length, colormap_type, image_type;
		unsigned short	colormap_index, colormap_length;
		unsigned char	colormap_size;
		unsigned short	x_origin, y_origin, width, height;
		unsigned char	pixel_size, attributes;
	} TargaHeader;

	TargaHeader		targa_header;
	dword_*			buffer;
	dword_*			pixbuff;

	FILE *fin = fopen( fileName.c_str(), "rb" );
	if( !fin )
		return NULL;

	targa_header.id_length = fgetc(fin);
	targa_header.colormap_type = fgetc(fin);
	targa_header.image_type = fgetc(fin);
	
	targa_header.colormap_index = fgetLittleShort(fin);
	targa_header.colormap_length = fgetLittleShort(fin);
	targa_header.colormap_size = fgetc(fin);
	targa_header.x_origin = fgetLittleShort(fin);
	targa_header.y_origin = fgetLittleShort(fin);
	width = targa_header.width = fgetLittleShort(fin);
	height = targa_header.height = fgetLittleShort(fin);
	targa_header.pixel_size = fgetc(fin);
	targa_header.attributes = fgetc(fin);

	if( targa_header.image_type !=2 )  // Only wants uncompressed, RGB images
	{
		fclose( fin );
		return NULL;
	}

	// Only open 24 or 32 bit targas
	if( targa_header.colormap_type !=0 || ( targa_header.pixel_size != 32 && targa_header.pixel_size != 24 ) )
	{
		fclose( fin );
		return NULL;
	}

	buffer = new dword_[ targa_header.width*targa_header.height ];
	assert( buffer );

	if( targa_header.id_length != 0 )
		fseek( fin, targa_header.id_length, SEEK_CUR );		// Skip TARGA image comment
	
	for( int row=targa_header.height-1; row>=0; row-- )
	{
		// Horisontal flip
		pixbuff = buffer + row*targa_header.width;

		for( int column=0; column<targa_header.width; column++ )
		{
			unsigned char red=0, green=0, blue=0, alphabyte=0;
			switch( targa_header.pixel_size )
			{
				case 24:
					blue = getc(fin);
					green = getc(fin);
					red = getc(fin);
					*pixbuff++ = 0xff000000 | (red<<16) | (green <<8) | blue;
					break;
				case 32:
					// Not yet tested...
					blue = getc(fin);
					green = getc(fin);
					red = getc(fin);
					alphabyte = getc(fin);
					*pixbuff++ = (alphabyte<<24) | (red<<16) | (green<<8) | blue;
					break;
				default:
					assert( 0 );
					break;
			}
		}
	}
	
	fclose( fin );

	return buffer;
}