// DDraw.cpp: implementation of the CDDraw class.
//
//////////////////////////////////////////////////////////////////////


#include "..\include\Draw.h"
#include <cassert>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDDraw::~CDDraw()
{
	if( m_pddsBackBuffer )
	{
		m_pddsBackBuffer->Release();
		m_pddsBackBuffer = NULL;
	}

	if( m_pddsPrimary )
	{
		m_pddsPrimary->Release();
		m_pddsPrimary = NULL;
	}

	if( m_pDD )
	{
		m_pDD->Release();
		m_pDD = NULL;
	}
}

CDDraw::CDDraw( HWND _hWnd, int _width, int _height )
{
	m_pDD = NULL;
	m_pddsPrimary = NULL;
	m_pddsBackBuffer = NULL;

#ifdef __SE_WINDOWED__
	InitWindowed( _hWnd, _width, _height );
#endif

#ifdef __SE_FULLSCREEN__
	InitFullscreen( _hWnd, _width, _height );
#endif
}

#ifdef __SE_WINDOWED__
void CDDraw::InitWindowed( HWND _hWnd, int _width, int _height )
{
    if( FAILED( DirectDrawCreateEx( NULL, (VOID**)&m_pDD, IID_IDirectDraw7, NULL ) ) )
		assert( 0 );

    if( FAILED( m_pDD->SetCooperativeLevel( _hWnd, DDSCL_NORMAL ) ) )
		assert( 0 );

	// Primary with no back buffers
	DDSURFACEDESC2 ddsd;
	ZeroMemory( &ddsd, sizeof(ddsd) );
	ddsd.dwSize  = sizeof( ddsd);
	ddsd.dwFlags = DDSD_CAPS;
	ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;

    if( FAILED( m_pDD->CreateSurface( &ddsd, &m_pddsPrimary, NULL ) ) )
		assert( 0 );

	// Create secondary surface
	ZeroMemory( &ddsd, sizeof(ddsd) );
	ddsd.dwSize  = sizeof( ddsd);
	ddsd.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH;
	ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN;
    ddsd.dwWidth  = _width;
    ddsd.dwHeight = _height;

    if( FAILED( m_pDD->CreateSurface( &ddsd, &m_pddsBackBuffer, NULL ) ) )
		assert( 0 );

    LPDIRECTDRAWCLIPPER pcClipper;
    if( FAILED( m_pDD->CreateClipper( 0, &pcClipper, NULL ) ) )
		assert( 0 );
 
    pcClipper->SetHWnd( 0, _hWnd );
    m_pddsPrimary->SetClipper( pcClipper );
    pcClipper->Release();
}
#endif

#ifdef __SE_FULLSCREEN__
void CDDraw::InitFullscreen( HWND _hWnd, int _width, int _height )
{
    if( FAILED( DirectDrawCreateEx( NULL, (VOID**)&m_pDD, IID_IDirectDraw7, NULL ) ) )
		assert( 0 );

    if( FAILED( m_pDD->SetCooperativeLevel( _hWnd, DDSCL_ALLOWREBOOT | DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN ) ) )
		assert( 0 );

	if( FAILED( m_pDD->SetDisplayMode( _width, _height, 32, 0, 0 ) ) )
		if( FAILED( m_pDD->SetDisplayMode( _width, _height, 16, 0, 0 ) ) )
			assert( 0 );

	// Primary with one back buffer
	DDSURFACEDESC2 ddsd;
	ZeroMemory( &ddsd, sizeof(ddsd) );
	ddsd.dwSize  = sizeof( ddsd);
	ddsd.dwFlags = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
	ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE | DDSCAPS_3DDEVICE | DDSCAPS_FLIP | DDSCAPS_COMPLEX;
	ddsd.dwBackBufferCount = 1;

    if( FAILED( m_pDD->CreateSurface( &ddsd, &m_pddsPrimary, NULL ) ) )
		assert( 0 );

	// Get a pointer to the back buffer
	DDSCAPS2 ddscaps;
	ZeroMemory(&ddscaps, sizeof(ddscaps));
	ddscaps.dwCaps = DDSCAPS_BACKBUFFER;

	if( FAILED( m_pddsPrimary->GetAttachedSurface( &ddscaps, &m_pddsBackBuffer ) ) )
		assert( 0 );
}
#endif
